/*
  Program: Zaifrovn textu podle jednoduchch ifer
  Autor: Radek Pelnek  
  Ploha ke knize "Jak to vyeit",  www.radekpelanek.cz/?jtv

  Zadn projektu: Napite program, kter bude schopen zaifrovat a deifrovat
  zprvu podle nkolika rznch princip (napklad substituce, transpozice,
  jednoduch kdovn). Vstupem programu je text k zaifrovn i deifrovn,
  volba ifrovacho systmu a ppadn volba hesla. Vstupem programu je
  transformovan text.

  Tento program realizuje jednoduchou zkladn funkcionalitu projektu -
  zaifrovn s vyuitm zkladnch ifer a kdovn.
*/


#include <iostream>
#include <string>
#include <cctype>

#define MAX 100

using namespace std;

// Pevede etzec na velk psmena.
// Vechny ostatn funkce pedpokldaj etzce tvoen velkmi psmeny.
string uppercase(string s) {
  string out;
  for (int i=0; i<s.length(); i++)
    if (isalpha(s[i]))
      out.push_back(toupper(s[i]));
  return out;
}

// Pevod etzce do vybranho kdovn.
string kod(string s, int kodovani) {
  // kodovani:
  // 1 = morse
  // 2 = brail
  string morse[26] = {".-","-...","-.-.","-..",".","..-.","--.","....","..",".---",
		 "-.-",".-..","--","-.","---",".--.","--.-",".-.","...","-",
		 "..-","...-",".--","-..-","-.--","--.."};
  string braille[26] = {"BWWWWW","BWBWWW","BBWWWW","BBWBWW","BWWBWW","BBBWWW",
			"BBBBWW","BWBBWW","WBBWWW","WBBBWW","BWWWBW","BWBWBW",
			"BBWWBW","BBWBBW","BWWBBW","BBBWBW","BBBBBW","BWBBBW",
			"WBBWBW","WBBBBW","BWWWBB","BWBWBB","WBBBWB","BBWWBB",
			"BBWBBB","BWWBBB"};
  string vystup;

  for (int i=0; i<s.length(); i++) {
    char c = s[i] - 'A';
    switch (kodovani) {
    case 1: vystup.append(morse[c]); break;
    case 2: vystup.append(braille[c]); break;
    }
    vystup.push_back('|');
  }
  return vystup;
}

// Pevod etzce na ASCII kd.
void print_ascii_hex(string s) {
  string vystup;

  for (int i=0; i<s.length(); i++) {
    char c = s[i];
    printf("%X,", c);
  }
  cout<<endl;
}

string posun(string s, int k) {
  string vystup;

  for (int i=0; i<s.length(); i++) {
    char c = s[i];
    c += k;
    if (c > (char) 'Z') c -= 26;
    vystup.push_back(c);
  }    
  return vystup;
}

string vigener(string s, string heslo) {
  string vystup;
  
  for (int i=0; i<s.length(); i++) {
    char c = s[i];
    char h = heslo[i % heslo.length()];
    c += h - ((char) 'A');
    if (c > (char) 'Z') c -= 26;
    vystup.push_back(c);
  }    
  
  return vystup;
}

string pozpatku(string s) {
  string vystup;
  
  string::reverse_iterator ri;
  for (ri=s.rbegin(); ri < s.rend(); ri++)
    vystup.push_back(*ri);
  
  return vystup;
}

string transpozice_obn(string s, int n) {
  string vystup = s;
  int x = 0, shift = 0;
  
  for (int i=0; i<s.length(); i++) {
    char c = s[i];
    vystup[x] = c;
    x = x + n;
    if (x >= s.length()) { shift++; x = shift; }
  }
  
  return vystup;
}

string transpozice_heslo(string s, string heslo) {
  string vystup;
  int n = heslo.length();
  int perm[MAX]; //permutace psmen v heslu
  int akt = 0, i;
  char c;
  for (c = 'A'; c <= 'Z'; c++) {
    for (i=0; i<n; i++)
      if (heslo[i] == c) { perm[akt] = i; akt++; }
  }
  while (s.length() % n != 0) s.push_back('X');

  // Kontroln vpis permutace.
  //  cout << "Perm: ";
  //  for (i=0; i<n;i++) cout << perm[i] << " ";
  //  cout <<endl;
  
  for (i=0; i<s.length(); i++) {
    vystup.push_back(s[ (i/n)*n + perm[i%n] ]);
  }    
  
  return vystup;
}

int main() {
  string zprava, heslo, vystup;
    
  cout << "Zadej zpravu:"<<endl;
  getline(cin, zprava);

  cout << "Zadej heslo:"<<endl;
  getline(cin, heslo);

  zprava = uppercase(zprava);
  heslo = uppercase(heslo);

  cout << "Upravena zprava: "<< zprava << endl;
  cout << "Upravene heslo:  "<< heslo << endl;

  cout << "Morse:"<<endl;
  cout << kod(zprava,1) <<endl;
  
  cout << "Braille:"<<endl;
  cout << kod(zprava,2) <<endl;  
  
  cout << "ASCII:"<<endl;
  print_ascii_hex(zprava);
  
  cout << "Posun o 3:"<<endl;
  cout << posun(zprava, 3) <<endl;

  cout << "Posun o heslo:"<<endl;
  cout << vigener(zprava, heslo) <<endl;

  cout << "Pozpatku:"<<endl;
  cout << pozpatku(zprava) <<endl;

  cout << "Transpozce ob 3:"<<endl;
  cout << transpozice_obn(zprava,3) <<endl;
  
  cout << "Transpozice podle hesla:"<<endl;
  cout << transpozice_heslo(zprava, heslo) <<endl;

  return 0;
}
